<!doctype html>
<title>Mitogen for Ansible (Redirect)</title>
<script>
    {% include "piwik-config.js" %}
    var u="https://networkgenomics.com/p/tr/";
    _paq.push(['setTrackerUrl', u+'ep']);
</script>
<script src="https://networkgenomics.com/p/tr/js"></script>
<script>
    setTimeout(function() {
        window.location = 'https://networkgenomics.com/ansible/';
    }, 0);
</script>
<meta http-equiv="Refresh" content="0; url=https://networkgenomics.com/ansible/">
