
body {
    font-size: 100%;
}

.sphinxsidebarwrapper {
    padding-top: 0 !important;
}

.sphinxsidebar {
    font-size: 80% !important;
}

.sphinxsidebar h3 {
    font-size: 130% !important;
}

img + p,
h1 + p,
h2 + p,
h3 + p,
h4 + p,
h5 + p
{
    margin-top: 0;
}

.section > h3:first-child {
    margin-top: 15px !important;
}

.body h1 { font-size: 200% !important; }
.body h2 { font-size: 165% !important; }
.body h3 { font-size: 125% !important; }
.body h4 { font-size: 110% !important; font-weight: bold; }
.body h5 { font-size: 100% !important; font-weight: bold; }

.body h1,
.body h2,
.body h3,
.body h4,
.body h5 {
    margin-top: 30px !important;
    color: #7f0000;
}

.body h1 {
    margin-top: 0 !important;
}

body,
.sphinxsidebar,
.sphinxsidebar h1,
.sphinxsidebar h2,
.sphinxsidebar h3,
.sphinxsidebar h4,
.sphinxsidebar h5,
.body h1,
.body h2,
.body h3,
.body h4,
.body h5 {
    /*font-family: sans-serif !important;*/
    font-family: -apple-system,BlinkMacSystemFont,Segoe UI,Helvetica,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol !important;
}


.document {
    width: 1000px !important;
}

div.figure {
    padding: 0;
}

div.body li {
    margin-bottom: 0.5em;
}


/*
 * Undo the hyphens: auto in Sphinx basic.css.
 */
div.body p, div.body dd, div.body li, div.body blockquote {
    -moz-hyphens: inherit;
    -ms-hyphens: inherit;
    -webkit-hyphens: inherit;
    hyphens: inherit;
}



/*
 * Setting :width; on an image causes Sphinx to turn the image into a link, so
 * set :Class: instead.
 */
.mitogen-full-width {
    width: 100%;
}

.mitogen-right-150 {
    float: right;
    padding-left: 8px;
    width: 150px;
}

.mitogen-right-180 {
    float: right;
    padding-left: 8px;
    width: 180px;
}

.mitogen-right-225 {
    float: right;
    padding-left: 8px;
    width: 225px;
}

.mitogen-right-275 {
    float: right;
    padding-left: 8px;
    width: 275px;
}

.mitogen-right-300 {
    float: right;
    padding-left: 8px;
    width: 300px;
}

.mitogen-right-350 {
    float: right;
    padding-left: 8px;
    width: 350px;
}

.mitogen-logo-wrap {
    shape-margin: 8px;
    shape-outside: polygon(
        100% 0, 50% 10%, 24% 24%, 0% 50%, 24% 75%, 50% 90%, 100% 100%
    );
}
